@@ -20,7 +20,7 @@
  * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
  * OTHER DEALINGS IN THE SOFTWARE. */
 
-#define LDBS_VERSION "0.5"	/* Version of the LDBS spec this library
+#define LDBS_VERSION "0.6"	/* Version of the LDBS spec this library
 				 * implements */
 
 /* Structures and functions to manage a LibDsk Block Store disk image, 
@@ -126,7 +126,7 @@ typedef enum
 	RECMODE_MASK       = 0x00FF,
 	RECMODE_MFM        = 0x0000,	
 	RECMODE_FM         = 0x0001,
-/* Recording modes 0x10-0x2F are GCR, defined as the GCR format byte 
+/* Recording modes 0x10-0x2F are Apple GCR, defined as the GCR format byte 
  * (byte 0x51 of an Apple Disk Copy file) masked with 0x1F, plus 0x10.
  * These fall into the range RECMODE_GCR_FIRST - RECMODE_GCR_LAST */
 	RECMODE_GCR_FIRST  = 0x0010,
@@ -134,6 +134,7 @@ typedef enum
 	RECMODE_GCR_PRODOS = 0x0014,      /* Apple IIgs Prodos 800k GCR */
 	RECMODE_GCR_LISA   = 0x0022,      /* Apple Lisa 400k GCR */
 	RECMODE_GCR_LAST   = 0x002F,
+	RECMODE_GCR_C64	   = 0x0030,	  /* Commodore 64 GCR */
 
 	/* High byte of dg_fm: Other data recording flags */
 	RECMODE_FLAGMASK = 0xFF00,
