@@ -58,7 +58,7 @@
 extern "C" {
 #endif
 
-#define LIBDSK_VERSION "1.5.18"
+#define LIBDSK_VERSION "1.5.21"
 
 /************************* TYPES ********************************/
 
@@ -161,6 +161,9 @@ typedef enum
 	FMT_1200K,	/* 15 sectors, 2 sides  */
 	FMT_MAC400,	/* Apple GCR 400k */
 	FMT_MAC800,	/* Apple GCR 800k */
+	FMT_C64_35,	/* Commodore C64 35 tracks */
+	FMT_C64_40,	/* Commodore C64 40 tracks */
+	FMT_C64_CPM,	/* Commodore C64 CP/M */
 	FMT_UNKNOWN = -1
 } dsk_format_t;
 
@@ -175,7 +178,7 @@ typedef enum
  * don't use 2 for any future recording mode in this structure; use
  * 3+ for MFM variants */
 
-/* Recording modes 0x10-0x2F are GCR, defined as the GCR format byte 
+/* Recording modes 0x10-0x2F are Apple GCR, defined as the GCR format byte 
  * (byte 0x51 of an Apple Disk Copy file) masked with 0x1F, plus 0x10.
  * These fall into the range RECMODE_GCR_FIRST - RECMODE_GCR_LAST */
 	RECMODE_GCR_FIRST  = 0x0010,
@@ -183,6 +186,7 @@ typedef enum
 	RECMODE_GCR_PRODOS = 0x0014,      /* Apple IIgs Prodos 800k GCR */
 	RECMODE_GCR_LISA   = 0x0022,      /* Apple Lisa 400k GCR */
 	RECMODE_GCR_LAST   = 0x002F,
+	RECMODE_GCR_C64    = 0x0030,	  /* Commodore 64 GCR */
 
 	/* High byte of dg_fm: Other data recording flags */
 	RECMODE_FLAGMASK   = 0xFF00,
@@ -309,6 +313,16 @@ LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_creat(DSK_PDRIVER *self, const char *filena
 			const char *type,
 			const char *compress);
 
+/* As for "open", but performs a diagnostic dump of the disk image to the
+ * output callback */
+LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_diagopen(DSK_PDRIVER *self, 
+			const char *filename, 
+			const char *type,
+			const char *compress,
+			DSK_REPORTFUNC diagfunc,
+			DSK_REPORTEND diagend);
+
+
 /* Close a DSK file. Frees the pointer and sets it to null. */
 
 LDPUBLIC32 dsk_err_t  LDPUBLIC16 dsk_close(DSK_PDRIVER *self);
