@@ -600,6 +600,13 @@ dsk_err_t tlzh_open(COMPRESS_DATA *self)
 		return DSK_ERR_NOTME;
 	}
 
+	if (magic[4] >= 0x0A && magic[4] <= 0x13)
+	{
+		fprintf(stderr, "TD0: 'Old Advanced' Teledisk compression is not supported.\n");
+		fclose(tlzh_self->fp_in);
+		return DSK_ERR_NOTME;
+	}
+
 	/* OK. This looks like an advanced Teledisk file. Decompress it to a 
 	 * normal Teledisk file. */
 	rewind(tlzh_self->fp_in);
