@@ -50,6 +50,9 @@ extern DRV_CLASS dc_remote;	/* All remote drivers */
 extern DRV_CLASS dc_gotek1440;	/* Gotek USB device (1.4Mb disc images) */
 extern DRV_CLASS dc_gotek720;	/* Gotek USB device (720k disc images) */
 extern DRV_CLASS dc_dc42;	/* Apple DiskCopy 4.2 */
+extern DRV_CLASS dc_d88;	/* D88 disk image */
+extern DRV_CLASS dc_d64;	/* D64 disk image */
+extern DRV_CLASS dc_d64cpm;	/* D64 CP/M disk image */
 #ifdef LINUXFLOPPY
 extern DRV_CLASS dc_linux;	/* Linux driver */
 #endif
