@@ -71,14 +71,17 @@ typedef struct adisk_recheader
 	unsigned short cylinder;	
 } ADISK_RECHEADER;
 
-static dsk_err_t adisk_readheader(FILE *fp, ADISK_RECHEADER *result)
+static dsk_err_t adisk_readheader(FILE *fp, ADISK_RECHEADER *result, 
+	DSK_REPORTFUNC diagfunc)
 {
 	unsigned char buf[16];
 	int len;
 
+	long fpos = ftell(fp);
+
 	len = fread(buf, 1, sizeof(buf), fp);
 	if (len < (int)sizeof(buf)) return DSK_ERR_SEEKFAIL;
-
+	
 	result->item_type   = ldbs_peek4(buf);
 	result->compression = ldbs_peek2(buf+4);
 	result->header_size = ldbs_peek2(buf+6);
@@ -87,14 +90,38 @@ static dsk_err_t adisk_readheader(FILE *fp, ADISK_RECHEADER *result)
 	result->sector      = buf[13];
 	result->cylinder    = ldbs_peek2(buf+14);
 
+	if (diagfunc)
+	{
+		char *blktype;
+
+		diaghead(diagfunc, "Block");
+		switch (result->item_type)
+		{
+			case APRIDISK_MAGIC: 	blktype = "Data"; break;
+			case APRIDISK_COMMENT:	blktype = "Comment"; break;
+			case APRIDISK_DELETED:	blktype = "Deleted"; break;
+			case APRIDISK_CREATOR:	blktype = "Creator"; break;
+			default:		blktype = "Unknown"; break;
+		}
+		diaghex(diagfunc, fpos + 0, buf + 0, 4, "Block type: %s", blktype);
+		diaghex(diagfunc, fpos + 4, buf + 4, 2, "Compression");
+		diaghex(diagfunc, fpos + 6, buf + 6, 2, "Header size");
+		diaghex(diagfunc, fpos + 8, buf + 8, 4, "Data size");
+		diaghex(diagfunc, fpos +12, buf +12, 1, "Head");
+		diaghex(diagfunc, fpos +13, buf +13, 1, "Sector");
+		diaghex(diagfunc, fpos +14, buf +14, 2, "Cylinder");
+	}
 	len = result->header_size - 16;
 
+	if (diagfunc && len)
+	{
+		diagrawrange(diagfunc, fp, fpos + 16, len, "Block data");
+	}
 	while (len)
 	{
 		if (fgetc(fp) == EOF) return DSK_ERR_SEEKFAIL;
 		--len;
 	}
-	
 	return DSK_ERR_OK;
 }
 
@@ -134,19 +161,20 @@ static size_t adisk_decompress(unsigned char *compressed, size_t c_len,
 
 /* Read the next block from the Apridisk file and add it to the 
  * blockstore */
-static dsk_err_t adisk_add_block(ADISK_DSK_DRIVER *self, FILE *fp)
+static dsk_err_t adisk_add_block(ADISK_DSK_DRIVER *self, FILE *fp, DSK_REPORTFUNC diagfunc)
 {
 	dsk_err_t err;
 	unsigned char *buf, *buf2;
 	char secid[4];
 	int n, nsec, allsame;
 	LDBLOCKID blkid;
+	long fpos;
 
 	/* Load the track header. */
 	ADISK_RECHEADER rh;
 	LDBS_TRACKHEAD *trkh;
 
-	err = adisk_readheader(fp, &rh); if (err) return DSK_ERR_OVERRUN;
+	err = adisk_readheader(fp, &rh, diagfunc); if (err) return DSK_ERR_OVERRUN;
 /*
 	printf("rh: item_type=%08x\n", rh.item_type);
 	printf("rh: compression=%04x\n", rh.compression);
@@ -177,6 +205,8 @@ static dsk_err_t adisk_add_block(ADISK_DSK_DRIVER *self, FILE *fp)
 	/* Skip over deleted data blocks */
 	if (rh.item_type == APRIDISK_DELETED)
 	{	
+		fpos = ftell(fp);
+		diagrawrange(diagfunc, fp, fpos, rh.data_size, "Deleted data");
 		if (fseek(fp, rh.data_size, SEEK_CUR)) return DSK_ERR_SYSERR;
 		return DSK_ERR_OK;
 	}
@@ -184,12 +214,17 @@ static dsk_err_t adisk_add_block(ADISK_DSK_DRIVER *self, FILE *fp)
 	buf[rh.data_size] = 0;
 	if (buf == NULL) return DSK_ERR_NOMEM;
 
+	fpos = ftell(fp);
 	/* Try to load the payload. If that fails, bail out */
 	if (fread(buf, 1, rh.data_size, fp) < rh.data_size)
 	{
 		dsk_free(buf);
 		return DSK_ERR_NOTME;	
 	}
+	diaghex(diagfunc, fpos, buf, rh.data_size, 
+		rh.compression == APRIDISK_COMPRESSED ? "Compressed data" :
+		"Uncompressed data");
+
 	/* If block is compressed, decompress it */
 	if (rh.compression == APRIDISK_COMPRESSED)
 	{
@@ -328,7 +363,7 @@ static dsk_err_t adisk_add_block(ADISK_DSK_DRIVER *self, FILE *fp)
 
 
 /* Open an Apridisk drive image and convert to LDBS */
-dsk_err_t adisk_open(DSK_DRIVER *self, const char *filename)
+dsk_err_t adisk_open(DSK_DRIVER *self, const char *filename, DSK_REPORTFUNC diagfunc)
 {
 	FILE *fp;
 	ADISK_DSK_DRIVER *adiskself;
@@ -361,6 +396,13 @@ dsk_err_t adisk_open(DSK_DRIVER *self, const char *filename)
 		fclose(fp);
 		return DSK_ERR_NOTME;
 	}
+	/* OK: Identified as Apridisk */
+	diaghead(diagfunc, "Apridisk disk image");
+	diaghex(diagfunc, 0, header, sizeof(adisk_wmagic), "Magic number");
+	diaghex(diagfunc, sizeof(adisk_wmagic), 
+			  header + sizeof(adisk_wmagic),
+			  sizeof(header) - sizeof(adisk_wmagic), "Unused");
+
 	/* Seek to end of header */
 	fseek(fp, 0x80, SEEK_SET);	
 	/* Keep a copy of the filename; when writing back, we will need it */
@@ -379,7 +421,7 @@ dsk_err_t adisk_open(DSK_DRIVER *self, const char *filename)
 	dsk_report("Loading APRIDISK file into memory");
 	while (!feof(fp))	
 	{
-		err = adisk_add_block(adiskself, fp);
+		err = adisk_add_block(adiskself, fp, diagfunc);
 		/* DSK_ERR_OVERRUN: End of file */
 		if (err == DSK_ERR_OVERRUN) 
 		{
