@@ -80,7 +80,7 @@ void be_poke4(unsigned char *dest, unsigned long val)
 }
 
 
-dsk_err_t dc42_open(DSK_DRIVER *self, const char *filename)
+dsk_err_t dc42_open(DSK_DRIVER *self, const char *filename, DSK_REPORTFUNC diagfunc)
 {
 	dsk_pcyl_t cylinder;
 	dsk_phead_t head;
@@ -98,6 +98,7 @@ dsk_err_t dc42_open(DSK_DRIVER *self, const char *filename)
 	unsigned long datalen, taglen;
 	dsk_err_t err;
 	long datapos, tagpos;
+	long fpos;
 
 	/* Sanity check: Is this meant for our driver? */
 	if (self->dr_class != &dc_dc42) return DSK_ERR_BADPTR;
@@ -150,6 +151,23 @@ dsk_err_t dc42_open(DSK_DRIVER *self, const char *filename)
 	datalen = be_peek4(header + 0x40);	
 	taglen  = be_peek4(header + 0x44);
 
+	diaghead(diagfunc, "DiskCopy 4.2 disk image");
+	if (is_macbin)
+	{
+		diagrawrange(diagfunc, fp, 0x00, 0x80, "MacBinary header");
+		fpos = 0x80;
+	}
+	else	fpos = 0;
+	diaghex(diagfunc, fpos, header, 1, "Description length");
+	diaghex(diagfunc, fpos +  1, header +  1, 63, "Description");
+	diaghex(diagfunc, fpos + 64, header + 64,  4, "Data bytes"); 
+	diaghex(diagfunc, fpos + 68, header + 68,  4, "Tag bytes"); 
+	diaghex(diagfunc, fpos + 72, header + 72,  4, "Data checksum"); 
+	diaghex(diagfunc, fpos + 76, header + 76,  4, "Tag checksum"); 
+	diaghex(diagfunc, fpos + 80, header + 80,  1, "Encoding"); 
+	diaghex(diagfunc, fpos + 81, header + 81,  1, "Sidedness / GCR type / MFM sector size"); 
+	diaghex(diagfunc, fpos + 82, header + 82,  2, "Magic number"); 
+
 	/* OK, we're ready to load the file. */
 	err = ldbs_new(&dcself->dc42_super.ld_store, NULL, LDBS_DSK_TYPE);
 	if (err)
@@ -260,6 +278,9 @@ dsk_err_t dc42_open(DSK_DRIVER *self, const char *filename)
 /* On a GCR variable-speed disk, vary the number of sectors */
 			if (geom.dg_fm == RECMODE_GCR_MAC) 
 				secs = dg_macspt(cylinder);
+
+			diaghead(diagfunc, "Cylinder %d head %d (%d sectors)",
+				cylinder, head, secs);
 	
 			trkh = ldbs_trackhead_alloc(secs);
 			trkh->datarate = (geom.dg_datarate == RATE_HD) ? 2 : 1;
@@ -289,6 +310,10 @@ dsk_err_t dc42_open(DSK_DRIVER *self, const char *filename)
 						fclose(fp);
 						return DSK_ERR_NOTME;
 					}
+					diagnl(diagfunc);
+					diaghex(diagfunc, tagpos,
+				  	     secbuf + secsize, 12, 
+					     "Sector %d tags", sector);
 					tagpos = ftell(fp);
 				}
 				/* Load sector body */
@@ -301,6 +326,8 @@ dsk_err_t dc42_open(DSK_DRIVER *self, const char *filename)
 					fclose(fp);
 					return DSK_ERR_NOTME;
 				}
+				diaghex(diagfunc, datapos, secbuf, secsize, 
+					"Sector %d data", sector);
 				datapos = ftell(fp);
 				trkh->sector[sector].id_cyl = cylinder;
 				trkh->sector[sector].id_head = head;
@@ -391,6 +418,9 @@ dsk_err_t dc42_open(DSK_DRIVER *self, const char *filename)
 						err = ldbs_putblock_d(dcself->dc42_super.ld_store, 
 							DC42_RSRC_BLOCK, rsrcbuf, rsrclen);
 					}
+					diagnl(diagfunc);
+					diaghex(diagfunc, rsrcaddr, rsrcbuf,
+						rsrclen, "Resource fork");
 					dsk_free(rsrcbuf);
 				}
 			}
