@@ -156,7 +156,7 @@ typedef struct
 
 
 static dsk_err_t imd_load_track(IMD_DSK_DRIVER *self, dsk_ltrack_t count, 
-	FILE *fp)
+	FILE *fp, DSK_REPORTFUNC diagfunc)
 {
 	/* Start by loading the track header: Fixed */
 	LDBS_TRACKHEAD *trkh;
@@ -164,8 +164,11 @@ static dsk_err_t imd_load_track(IMD_DSK_DRIVER *self, dsk_ltrack_t count,
 	dsk_err_t err;
 	int n, psh, c, status;
 	unsigned short datalen[256];
+	long fpos;
+	unsigned char ch;
 
 	/* Load the track header: 5 bytes */
+	fpos = ftell(fp);
 	if (fread(&tmp.imdt_mode, 1, 4, fp) < 4)
 	{
 		return DSK_ERR_OVERRUN;	/* EOF */
@@ -175,6 +178,7 @@ static dsk_err_t imd_load_track(IMD_DSK_DRIVER *self, dsk_ltrack_t count,
 	{
 		return DSK_ERR_OVERRUN;	/* EOF */
 	}
+	ch = psh;
 	if (psh == 0xFF) 
 	{
 		tmp.imdt_seclen = 0xFFFF;
@@ -185,6 +189,17 @@ static dsk_err_t imd_load_track(IMD_DSK_DRIVER *self, dsk_ltrack_t count,
 		tmp.imdt_seclen = dsk_expand_psh(psh);
 		for (n = 0; n < 256; n++) datalen[n] = tmp.imdt_seclen;
 	}
+	diagnl(diagfunc);
+	diaghead(diagfunc, "Header for track %ld", (long)count);
+	diaghex (diagfunc, fpos, &tmp.imdt_mode, 1, "Recording mode");
+	diaghex (diagfunc, fpos + 1, &tmp.imdt_cylinder, 1, "Cylinder = %d",
+			tmp.imdt_cylinder);
+	diaghex (diagfunc, fpos + 2, &tmp.imdt_head, 1, "Head = %d",
+			tmp.imdt_head);
+	diaghex (diagfunc, fpos + 3, &tmp.imdt_sectors, 1, "Sectors = %d",
+			tmp.imdt_sectors);
+	diaghex (diagfunc, fpos + 4, &ch, 1, "Sector size = %d",
+			tmp.imdt_seclen);
 
 	/* Allocate an LDBS track header for that number of sectors */
 	trkh = ldbs_trackhead_alloc(tmp.imdt_sectors);	
@@ -221,6 +236,7 @@ static dsk_err_t imd_load_track(IMD_DSK_DRIVER *self, dsk_ltrack_t count,
 		trkh->sector[n].datalen = 128 << psh;
 	}
 	/* Load sector IDs */	
+	diagrawrange(diagfunc, fp, fpos + 5, tmp.imdt_sectors, "Sector IDs");
 	for (n = 0; n < tmp.imdt_sectors; n++)
 	{
 		c = fgetc(fp); 	
@@ -234,6 +250,8 @@ static dsk_err_t imd_load_track(IMD_DSK_DRIVER *self, dsk_ltrack_t count,
 	/* Load sector cylinder map (if present) */
 	if (tmp.imdt_head & 0x80)
 	{
+		fpos = ftell(fp);
+		diagrawrange(diagfunc, fp, fpos, tmp.imdt_sectors, "Sector cylinder map");
 		for (n = 0; n < tmp.imdt_sectors; n++)
 		{
 			c = fgetc(fp); 	
@@ -249,6 +267,8 @@ static dsk_err_t imd_load_track(IMD_DSK_DRIVER *self, dsk_ltrack_t count,
 	/* Load sector head map (if present) */
 	if (tmp.imdt_head & 0x40)
 	{
+		fpos = ftell(fp);
+		diagrawrange(diagfunc, fp, fpos, tmp.imdt_sectors, "Sector head map");
 		for (n = 0; n < tmp.imdt_sectors; n++)
 		{
 			c = fgetc(fp); 	
@@ -264,6 +284,8 @@ static dsk_err_t imd_load_track(IMD_DSK_DRIVER *self, dsk_ltrack_t count,
 	if (tmp.imdt_seclen == 0xFFFF)
 	{
 		int l, h;
+		fpos = ftell(fp);
+		diagrawrange(diagfunc, fp, fpos, tmp.imdt_sectors * 2, "Sector lengths");
 
 		for (n = 0; n < tmp.imdt_sectors; n++)
 		{
@@ -289,12 +311,15 @@ static dsk_err_t imd_load_track(IMD_DSK_DRIVER *self, dsk_ltrack_t count,
 			tmp.imdt_head & 0x3F, trkh->sector[n].id_sec);
 		
 		/* Get status for sector */
+		fpos = ftell(fp);
 		c = fgetc(fp); 	
 		if (c == EOF) 
 		{
 			ldbs_free(trkh);
 			return DSK_ERR_SYSERR;
 		}
+		ch = c;
+		diaghex(diagfunc, fpos, &ch, 1, "Sector %d status", n);
 		status = c;
 		/* Convert status to 8272 register bits */
 		switch(status)
@@ -336,6 +361,8 @@ static dsk_err_t imd_load_track(IMD_DSK_DRIVER *self, dsk_ltrack_t count,
 				trkh->sector[n].blockid = LDBLOCKID_NULL;
 				trkh->sector[n].copies = 0;
 				trkh->sector[n].filler = c;
+				ch = c;
+				diaghex(diagfunc, fpos + 1, &ch, 1, "Sector %d compressed", n);
 				break;
 			/* Uncompressed */
 			case ST_NORMAL: 		
@@ -356,6 +383,7 @@ static dsk_err_t imd_load_track(IMD_DSK_DRIVER *self, dsk_ltrack_t count,
 					ldbs_free(trkh);
 					return DSK_ERR_SYSERR;
 				}
+				diaghex(diagfunc, fpos + 1, buf, datalen[n], "Sector %d uncompressed", n);
 				err = ldbs_putblock(self->imd_super.ld_store, 
 					&trkh->sector[n].blockid, 
 					secid, buf, datalen[n]);
@@ -377,12 +405,13 @@ static dsk_err_t imd_load_track(IMD_DSK_DRIVER *self, dsk_ltrack_t count,
 
 
 
-dsk_err_t imd_open(DSK_DRIVER *self, const char *filename)
+dsk_err_t imd_open(DSK_DRIVER *self, const char *filename, DSK_REPORTFUNC diagfunc)
 {
 	FILE *fp;
 	IMD_DSK_DRIVER *imdself;
 	dsk_err_t err;	
 	int ccmt;
+	long fpos;
 	dsk_ltrack_t count = 0;
 	char *comment, *ucomment;
 	int termch;
@@ -431,6 +460,10 @@ dsk_err_t imd_open(DSK_DRIVER *self, const char *filename)
 		fclose(fp);
 		return DSK_ERR_NOTME;
 	}
+	diaghead(diagfunc, "IMD disk image");
+	diaghex(diagfunc, 0, (unsigned char *)comment, ccmt, 
+		"Signature / date stamp");
+
 	dsk_free(comment);
 
 	err = ldbs_new(&imdself->imd_super.ld_store, NULL, LDBS_DSK_TYPE);
@@ -445,6 +478,7 @@ dsk_err_t imd_open(DSK_DRIVER *self, const char *filename)
 	{
 		int n;
 
+		fpos = ftell(fp);
 		err = imd_readto(fp, 0x1A, 0x1A, &ccmt, &termch);
 		if (err)
 		{
@@ -463,6 +497,8 @@ dsk_err_t imd_open(DSK_DRIVER *self, const char *filename)
 			fclose(fp);
 			return DSK_ERR_SYSERR;
 		}	
+		diaghex(diagfunc, fpos, (unsigned char *)comment, ccmt, 
+			"Comment");
 		comment[ccmt - 1] = 0;
 		n = cp437_to_utf8(comment, NULL, -1);
 		ucomment = dsk_malloc(n);
@@ -498,7 +534,7 @@ dsk_err_t imd_open(DSK_DRIVER *self, const char *filename)
 
 	while (!feof(fp))
 	{
-		err = imd_load_track(imdself, count, fp);
+		err = imd_load_track(imdself, count, fp, diagfunc);
 		if (err == DSK_ERR_OVERRUN) 	/* EOF */
 		{
 			dsk_report_end();
