@@ -164,7 +164,8 @@ static dsk_err_t check_geom(LINUX_DSK_DRIVER *self, const DSK_GEOMETRY *dg)
 }
 
 
-dsk_err_t linux_open(DSK_DRIVER *self, const char *filename)
+dsk_err_t linux_open(DSK_DRIVER *self, const char *filename,
+		DSK_REPORTFUNC diagfunc)
 {
 	LINUX_DSK_DRIVER *lxself;
 	struct stat st;
@@ -206,7 +207,7 @@ dsk_err_t linux_open(DSK_DRIVER *self, const char *filename)
  * unless we wanted to mess around with mknod() here */
 dsk_err_t linux_creat(DSK_DRIVER *self, const char *filename)
 {
-	return linux_open(self, filename);
+	return linux_open(self, filename, NULL);
 }
 
 
