@@ -1,7 +1,7 @@
 /***************************************************************************
  *                                                                         *
  *    LIBDSK: General floppy and diskimage access library                  *
- *    Copyright (C) 2001  John Elliott <jce@seasip.demon.co.uk>            *
+ *    Copyright (C) 2001  John Elliott <seasip.webmaster@gmail.com>            *
  *                                                                         *
  *    This library is free software; you can redistribute it and/or        *
  *    modify it under the terms of the GNU Library General Public          *
@@ -33,9 +33,10 @@ typedef struct
 	int	     lx_forcehead;
 	int          lx_doublestep;
 	dsk_pcyl_t   lx_cylinder;
+	unsigned char lx_status[4];
 } LINUX_DSK_DRIVER;
 
-dsk_err_t linux_open(DSK_DRIVER *self, const char *filename);
+dsk_err_t linux_open(DSK_DRIVER *self, const char *filename, DSK_REPORTFUNC diagfunc);
 dsk_err_t linux_creat(DSK_DRIVER *self, const char *filename);
 dsk_err_t linux_close(DSK_DRIVER *self);
 dsk_err_t linux_read(DSK_DRIVER *self, const DSK_GEOMETRY *geom,
