@@ -68,7 +68,7 @@ DRV_CLASS dc_nwasp =
 	nwasp_from_ldbs	/* import as LDBS */
 };
 
-dsk_err_t nwasp_open(DSK_DRIVER *self, const char *filename)
+dsk_err_t nwasp_open(DSK_DRIVER *self, const char *filename, DSK_REPORTFUNC diagfunc)
 {
 	NWASP_DSK_DRIVER *nwself;
 	
@@ -90,6 +90,9 @@ dsk_err_t nwasp_open(DSK_DRIVER *self, const char *filename)
         if (fseek(nwself->nw_fp, 0, SEEK_END)) return DSK_ERR_SYSERR;
         nwself->nw_filesize = ftell(nwself->nw_fp);
 
+	diaghead(diagfunc, "NanoWasp disk image");
+	diagraw(diagfunc, nwself->nw_fp, "Raw data");
+
 	return DSK_ERR_OK;
 }
 
