@@ -41,7 +41,7 @@ DRV_CLASS dc_posixalt =
 	"Raw file driver (alternate sides) ",
 	posix_openalt,	/* open */
 	posix_creatalt,	/* create new */
-	posix_close,	/* close */
+	posix_closedsk,	/* close */
 	posix_read,	/* read sector, working from physical address */
 	posix_write,	/* write sector, working from physical address */
 	posix_format,	/* format track, physical */
@@ -70,7 +70,7 @@ DRV_CLASS dc_posixoo =
 	"Raw file driver (out and out) ",
 	posix_openoo,	/* open */
 	posix_creatoo,	/* create new */
-	posix_close,	/* close */
+	posix_closedsk,	/* close */
 	posix_read,	/* read sector, working from physical address */
 	posix_write,	/* write sector, working from physical address */
 	posix_format,	/* format track, physical */
@@ -99,7 +99,7 @@ DRV_CLASS dc_posixob =
 	"Raw file driver (out and back) ",
 	posix_openob,	/* open */
 	posix_creatob,	/* create new */
-	posix_close,	/* close */
+	posix_closedsk,	/* close */
 	posix_read,	/* read sector, working from physical address */
 	posix_write,	/* write sector, working from physical address */
 	posix_format,	/* format track, physical */
@@ -129,7 +129,8 @@ DRV_CLASS dc_posixob =
 
 
 
-dsk_err_t posix_open(DSK_DRIVER *self, const char *filename, dsk_sides_t s)
+dsk_err_t posix_open(DSK_DRIVER *self, const char *filename, dsk_sides_t s,
+	DSK_REPORTFUNC diagfunc)
 {
 	POSIX_DSK_DRIVER *pxself;
 	
@@ -151,23 +152,28 @@ dsk_err_t posix_open(DSK_DRIVER *self, const char *filename, dsk_sides_t s)
         if (fseek(pxself->px_fp, 0, SEEK_END)) return DSK_ERR_SYSERR;
         pxself->px_filesize = ftell(pxself->px_fp);
 
+	diaghead(diagfunc, "Raw / POSIX disk image");
+	diagraw(diagfunc, pxself->px_fp, "Raw data");
 	return DSK_ERR_OK;
 }
 
 
-dsk_err_t posix_openalt(DSK_DRIVER *self, const char *filename)
+dsk_err_t posix_openalt(DSK_DRIVER *self, const char *filename,
+	DSK_REPORTFUNC diagfunc)
 {
-	return posix_open(self, filename, SIDES_ALT);
+	return posix_open(self, filename, SIDES_ALT, diagfunc);
 }
 
-dsk_err_t posix_openoo(DSK_DRIVER *self, const char *filename)
+dsk_err_t posix_openoo(DSK_DRIVER *self, const char *filename,
+	DSK_REPORTFUNC diagfunc)
 {
-	return posix_open(self, filename, SIDES_OUTOUT);
+	return posix_open(self, filename, SIDES_OUTOUT, diagfunc);
 }
 
-dsk_err_t posix_openob(DSK_DRIVER *self, const char *filename)
+dsk_err_t posix_openob(DSK_DRIVER *self, const char *filename,
+	DSK_REPORTFUNC diagfunc)
 {
-	return posix_open(self, filename, SIDES_OUTBACK);
+	return posix_open(self, filename, SIDES_OUTBACK, diagfunc);
 }
 
 dsk_err_t posix_creat(DSK_DRIVER *self, const char *filename, dsk_sides_t s)
@@ -201,7 +207,7 @@ dsk_err_t posix_creatob(DSK_DRIVER *self, const char *filename)
 }
 
 
-dsk_err_t posix_close(DSK_DRIVER *self)
+dsk_err_t posix_closedsk(DSK_DRIVER *self)
 {
 	POSIX_DSK_DRIVER *pxself;
 
