@@ -467,17 +467,27 @@ static dsk_err_t rcpmfs_dump_options(RCPMFS_DSK_DRIVER *self, FILE *fp)
 
 
 
-static dsk_err_t rcpmfs_parse(RCPMFS_DSK_DRIVER *self, FILE *fp)
+static dsk_err_t rcpmfs_parse(RCPMFS_DSK_DRIVER *self, FILE *fp, DSK_REPORTFUNC diagfunc)
 {
 	char *s;
 	char linebuf[160];
+	char diagbuf[160];
 	int got_format = 0;
 	dsk_err_t err;
 
 	if (!self) return DSK_ERR_BADPTR;
 
+	diaghead(diagfunc, ".libdsk.ini file");
 	while (fgets(linebuf, sizeof(linebuf), fp))
 	{
+		if (diagfunc)
+		{
+			strcpy(diagbuf, linebuf);
+/* Trim off newlines */
+			s = strrchr(diagbuf, '\n');
+			if (s) *s = 0;
+			(*diagfunc)(diagbuf);
+		}
 /* Force case-insensitivity */
 		for (s = linebuf; *s; s++)
 		{
@@ -1234,7 +1244,7 @@ RTRACE(("Disc full, rolling back this entry\n"));
 
 
 
-dsk_err_t rcpmfs_open(DSK_DRIVER *self, const char *passed)
+dsk_err_t rcpmfs_open(DSK_DRIVER *self, const char *passed, DSK_REPORTFUNC diagfunc)
 {
 	dsk_err_t err;
 	struct stat st;
@@ -1294,11 +1304,57 @@ dsk_err_t rcpmfs_open(DSK_DRIVER *self, const char *passed)
 	fp = fopen(filename, "r");
 	if (fp)
 	{
-		err = rcpmfs_parse(rcself, fp);
+		err = rcpmfs_parse(rcself, fp, diagfunc);
 		fclose(fp);
 		if (err) return err;
 	}
-	return rcpmfs_readdir(rcself);
+	err = rcpmfs_readdir(rcself);
+	if (err) return err;
+	if (diagfunc)
+	{
+		unsigned char *buf;
+		dsk_pcyl_t cyl;
+		dsk_phead_t head;
+		dsk_psect_t sec;
+		dsk_lsect_t lsect, lsmax, lsboot;
+		long pos = 0;
+
+		buf = dsk_malloc(rcself->rc_geom.dg_secsize);
+		if (buf)
+		{
+			lsmax = rcself->rc_geom.dg_cylinders *
+				rcself->rc_geom.dg_sectors *
+				rcself->rc_geom.dg_heads;
+			lsboot = rcself->rc_systracks * 
+				  rcself->rc_geom.dg_sectors;
+
+			for (lsect = 0; lsect < lsmax; lsect++)
+			{
+				dg_ls2ps(&rcself->rc_geom, lsect,
+						&cyl, &head, &sec);
+				
+				if (!rcpmfs_read(self, &rcself->rc_geom,
+					buf, cyl, head, sec)) 
+				{
+					char *caption = "";
+
+					diaghead(diagfunc, "Simulated "
+						"cylinder %d head %d sector %d",
+						 cyl, head, sec);
+					if (lsect == lsboot) caption = "Simulated directory";
+					else if (lsect == 0) caption = "Simulated boot track"; 
+					diaghex(diagfunc, pos, buf, 
+						rcself->rc_geom.dg_secsize, 
+						caption);	
+					pos += rcself->rc_geom.dg_secsize;
+				}
+			}
+			dsk_free(buf);
+		}
+	}
+
+
+	return DSK_ERR_OK;
 }
 
 
@@ -1370,7 +1426,7 @@ dsk_err_t rcpmfs_creat(DSK_DRIVER *self, const char *passed)
 	fp = fopen(filename, "r");
 	if (fp)
 	{
-		err = rcpmfs_parse(rcself, fp);
+		err = rcpmfs_parse(rcself, fp, NULL);
 		fclose(fp);
 		if (err) return err;
 	}
