@@ -80,7 +80,7 @@ DRV_CLASS dc_simh =
 
 
 
-dsk_err_t simh_open(DSK_DRIVER *self, const char *filename)
+dsk_err_t simh_open(DSK_DRIVER *self, const char *filename, DSK_REPORTFUNC diagfunc)
 {
 	SIMH_DSK_DRIVER *simh_self;
 	
@@ -113,6 +113,9 @@ dsk_err_t simh_open(DSK_DRIVER *self, const char *filename)
 	dsk_isetoption(self, "FS:CP/M:CKS", 0x40, 1);
 	dsk_isetoption(self, "FS:CP/M:OFF", 6, 1);
 
+	diaghead(diagfunc, "SIMH disk image");
+	diagraw(diagfunc, simh_self->simh_fp, "Raw data");
+
 	return DSK_ERR_OK;
 }
 
