@@ -71,10 +71,6 @@ DRV_CLASS dc_tele =
 	tele_close,
 };
 
-/* #define MONITOR(x) printf x */    
-
-#define MONITOR(x) 
-
 
 #define DC_CHECK(s) \
 	TELE_DSK_DRIVER *self; \
@@ -109,7 +105,8 @@ static dsk_err_t tele_fread(TELE_DSK_DRIVER *self, tele_byte *buf, int count)
 /* Expand a compressed Teledisk sector to an LDBS sector, updating its
  * entry in the track header trkh. */
 static dsk_err_t convert_sector(TELE_DSK_DRIVER *self, LDBS_TRACKHEAD *trkh,
-				dsk_pcyl_t cyl, dsk_phead_t head, unsigned nsec)
+				dsk_pcyl_t cyl, dsk_phead_t head, unsigned nsec,
+				DSK_REPORTFUNC diagfunc)
 {
 	dsk_err_t err;
 	tele_byte buf[6];
@@ -120,8 +117,11 @@ static dsk_err_t convert_sector(TELE_DSK_DRIVER *self, LDBS_TRACKHEAD *trkh,
 	tele_byte *secbuf;
 	int allsame; 
 	char secid[4];
+	unsigned long fpos;
 
-	MONITOR (("Loading sector cyl=%d head=%d nsec=%d\n", cyl, head, nsec));
+	diaghead(diagfunc, "Sector header: Cylinder %d  Head %d  Sector %d",
+			cyl, head, nsec);
+	fpos = ftell(self->tele_fp);
 	err = tele_fread(self, buf, 6);
 	if (err) return err;
 	trkh->sector[nsec].id_cyl  = buf[0];
@@ -132,8 +132,13 @@ static dsk_err_t convert_sector(TELE_DSK_DRIVER *self, LDBS_TRACKHEAD *trkh,
 	syndrome = buf[4];
 	/* buf[5] is the CRC, ignored on load */
 
-	MONITOR(("ID=(%d,%d,%d) size=%d syndrome=0x%02x\n",
-			buf[0], buf[1], buf[2], dsk_expand_psh(buf[3]), syndrome));
+	diaghex(diagfunc, fpos + 0, buf + 0, 1, "ID: Cylinder");
+	diaghex(diagfunc, fpos + 1, buf + 1, 1, "ID: Head");
+	diaghex(diagfunc, fpos + 2, buf + 2, 1, "ID: Sector");
+	diaghex(diagfunc, fpos + 3, buf + 3, 1, "ID: Sector size");
+	diaghex(diagfunc, fpos + 4, buf + 4, 1, "Syndrome");
+	diaghex(diagfunc, fpos + 5, buf + 5, 1, "Header CRC");
+
 	/* Convert syndrome byte to error flags */
 	if (syndrome & 2) trkh->sector[nsec].st2 |= 0x20; /* CRC error */
 	if (syndrome & 4) trkh->sector[nsec].st2 |= 0x40; /* Control mark */
@@ -151,6 +156,8 @@ static dsk_err_t convert_sector(TELE_DSK_DRIVER *self, LDBS_TRACKHEAD *trkh,
 	/* Otherwise sector data follow */
 	err = tele_fread(self, buf, 3);
 	if (err) return err;
+	diaghex(diagfunc, fpos + 6, buf + 0, 2, "Sector data length");
+	diaghex(diagfunc, fpos + 8, buf + 2, 1, "Sector data encoding");
 
 	encoding   = buf[2];
 
@@ -158,19 +165,25 @@ static dsk_err_t convert_sector(TELE_DSK_DRIVER *self, LDBS_TRACKHEAD *trkh,
 	secbuf = dsk_malloc(ulen + 2);	/* In case of overflow */
 	if (!secbuf) return DSK_ERR_NOMEM;
 
-	MONITOR(("Compressed len = %d encoding=0x%02x ulen=%ld\n", 
-		ldbs_peek2(buf), encoding, ulen));
+//	MONITOR(("Compressed len = %d encoding=0x%02x ulen=%ld\n", 
+//		ldbs_peek2(buf), encoding, ulen));
 
 	switch(encoding)
 	{
 		case 0:	/* Uncompressed */
 			err = tele_fread(self, secbuf, ulen);
 			if (err) { dsk_free(secbuf); return err; }
+			diaghex(diagfunc, fpos + 9, secbuf, ulen, 
+				"Uncompressed sector data");
 			break;
 		case 1:	/* One pattern */
 			err = tele_fread(self, pattern, 4);
 			if (err) { dsk_free(secbuf); return err; }
 
+			diaghex(diagfunc, fpos + 9, pattern, 2, 
+				"RLE1: Repetitions = %d", ldbs_peek2(pattern));
+			diaghex(diagfunc, fpos + 11, pattern + 2, 2, 
+				"RLE1: Pattern");
 			for (n = 0; n < ldbs_peek2(pattern) && n*2 < ulen; n++)
 			{
 				secbuf[n*2]   = pattern[2];
@@ -185,16 +198,23 @@ static dsk_err_t convert_sector(TELE_DSK_DRIVER *self, LDBS_TRACKHEAD *trkh,
 				tele_byte ptype, plen;
 
 				wleft = ulen - pos;
+				if (diagfunc)
+				{
+					fpos = ftell(self->tele_fp);
+				}
 				err = tele_fread(self, pattern, 2);
 				if (err) { dsk_free(secbuf); return err; }
 				/* Load pattern type & length */
 				ptype = pattern[0];
 				plen = pattern[1];
+				diaghex(diagfunc, fpos,  pattern, 1, "RLE2: Type");
+				diaghex(diagfunc, fpos+1, pattern+1, 1, "RLE2: Length");
 				/* Uncompressed run? */
 				if (ptype == 0)
 				{
 					err = tele_fread(self, pattern, plen);
 					if (err) { dsk_free(secbuf); return err; }
+					diaghex(diagfunc, fpos+2, pattern, plen, "RLE2: Uncompressed data");
 					if (plen > wleft) plen = wleft;
 					memcpy(secbuf + pos, pattern, plen);
 					pos += plen;
@@ -203,6 +223,7 @@ static dsk_err_t convert_sector(TELE_DSK_DRIVER *self, LDBS_TRACKHEAD *trkh,
 				/*  Compressed run */
 				err = tele_fread(self, pattern, (1 << ptype));
 				if (err) { dsk_free(secbuf); return err; }
+				diaghex(diagfunc, fpos+2, pattern, 1 << ptype, "RLE2: Compressed data");
 				for (n = 0; n < plen; n++)
 				{
 /* Ensure the amount of data written does not exceed len */
@@ -251,11 +272,15 @@ static dsk_err_t convert_sector(TELE_DSK_DRIVER *self, LDBS_TRACKHEAD *trkh,
 
 
 /* Open a Teledisk file and load it into the blockstore */
-dsk_err_t tele_open(DSK_DRIVER *s, const char *filename)
+dsk_err_t tele_open(DSK_DRIVER *s, const char *filename,
+		DSK_REPORTFUNC diagfunc)
 {
 	dsk_err_t err;
 	unsigned short crc;
 	unsigned char header[12];
+	unsigned long fpos;
+	int all_fm = 0;
+
 	DC_CHECK(s);
 
 	self->tele_fp = fopen(filename, "r+b");
@@ -280,6 +305,8 @@ dsk_err_t tele_open(DSK_DRIVER *s, const char *filename)
 	memset(&self->tele_head, 0, sizeof(self->tele_head));
 	memcpy(self->tele_head.magic, header, 2);
 	self->tele_head.magic[2] = 0;
+
+
 	self->tele_head.volume_seq  = header[2];
 	self->tele_head.volume_id   = header[3];
 	self->tele_head.ver         = header[4];
@@ -297,7 +324,33 @@ dsk_err_t tele_open(DSK_DRIVER *s, const char *filename)
 		fclose(self->tele_fp);
 		return DSK_ERR_NOTME;
 	}
-
+/* Now we're reasonably sure it's Teledisk, do a diagnostic dump if required */
+	diaghead(diagfunc, "Teledisk file header");
+	diaghex(diagfunc, 0, header + 0, 2, "Magic number");
+	diaghex(diagfunc, 2, header + 2, 1, "Volume sequence");	
+	diaghex(diagfunc, 3, header + 3, 1, "Volume ID");	
+	diaghex(diagfunc, 4, header + 4, 1, "Version");	
+	if (header[5] & 0x80)
+	{
+		diaghex(diagfunc, 5, header + 5, 1, "Data rate  all tracks FM");
+		all_fm = 1;
+	}
+	else
+	{
+		diaghex(diagfunc, 5, header + 5, 1, "Data rate");	
+	}
+	diaghex(diagfunc, 6, header + 6, 1, "Drive type");	
+	if (header[7] & 0x80)
+	{	
+		diaghex(diagfunc, 7, header + 7, 1, "Double track (comment present)");	
+	}
+	else
+	{
+		diaghex(diagfunc, 7, header + 7, 1, "Double track");	
+	}
+	diaghex(diagfunc, 8, header + 8, 1, "DOS mode");	
+	diaghex(diagfunc, 9, header + 9, 1, "Sides");	
+	diaghex(diagfunc,10, header +10, 2, "Header CRC");	
 	/* Advanced compression not handled here -- see comptlzh.c */
 	if (!strcmp((char *)header, "td"))
 	{
@@ -322,12 +375,25 @@ dsk_err_t tele_open(DSK_DRIVER *s, const char *filename)
 		char *comment_data;
 		char *ucomment;
 
+		fpos = ftell(self->tele_fp);
+	
 		/* Load comment header */
 		if (tele_fread(self, header, 10))
 		{
 			fclose(self->tele_fp);
 			return DSK_ERR_SYSERR;
 		}
+		diaghead(diagfunc, "Comment record");
+		diaghex(diagfunc, fpos, header, 2,  "Comment CRC");
+		diaghex(diagfunc, fpos + 2, header + 2, 2,  "Comment length");
+		diaghex(diagfunc, fpos + 4, header + 4, 3, 
+			"Date: %04d-%02d-%02d", 
+			header[4]+1900, 1+(header[5] % 12), header[6] % 100);
+
+		diaghex(diagfunc, fpos + 7, header + 7, 3, 
+			"Time: %02d:%02d:%02d", 
+			header[7] % 100, header[8] % 100, header[9] % 100);
+
 		comment_len  = ldbs_peek2(header + 2);
 		comment_data = dsk_malloc(comment_len + 1);
 		if (!comment_data)
@@ -343,6 +409,8 @@ dsk_err_t tele_open(DSK_DRIVER *s, const char *filename)
 			fclose(self->tele_fp);
 			return DSK_ERR_SYSERR;
 		}
+		diaghex(diagfunc, fpos + 10, (unsigned char *)comment_data, 
+				comment_len, "Comment");	
 		/* 0-terminate the loaded comment */
 		comment_data[comment_len] = 0;
 		/* Replace double-zeroes with CR/LF pairs */
@@ -406,6 +474,7 @@ dsk_err_t tele_open(DSK_DRIVER *s, const char *filename)
 
 		/* Try to read the track header. If it isn't present because
 		 * of EOF, fine. */
+		fpos = ftell(self->tele_fp);
 		if (tele_fread(self, header, 4))
 		{
 			if (feof(self->tele_fp)) break;
@@ -414,8 +483,27 @@ dsk_err_t tele_open(DSK_DRIVER *s, const char *filename)
 			fclose(self->tele_fp);
 			return err;
 		}
+		diaghead(diagfunc, "Track header");
+		diaghex(diagfunc, fpos, header, 1, 
+			"Track header: SPT      = %3d",(signed char)(header[0]));
 		/* header[0] = sectors / track, if 0xFF then break */
 		if (header[0] == 0xFF) break;
+		diaghex(diagfunc, fpos + 1, header + 1, 1, 
+			"Track header: Cylinder = %3d", header[1]);
+		if (all_fm)
+		{
+			diaghex(diagfunc, fpos + 2, header + 2, 1, 
+				"Track header: Head     = %3d", 
+				header[2] & 0x7F);
+		}
+		else
+		{
+			diaghex(diagfunc, fpos + 2, header + 2, 1, 
+				"Track header: Head     = %3d %s", 
+				header[2] & 0x7F, (header[2] & 0x80) ? "FM" : "MFM");
+		}
+		diaghex(diagfunc, fpos + 3, header + 3, 1, 
+			"Track header: CRC");
 		cyl = header[1];
 		head = header[2] & 0x7F;
 		trkh = ldbs_trackhead_alloc(header[0]);
@@ -432,7 +520,7 @@ dsk_err_t tele_open(DSK_DRIVER *s, const char *filename)
 		}
 		/* Recording mode may be indicated in disk or track
 		 * header. */
-		if ((self->tele_head.datarate & 0x80) || header[2] & 0x80)
+		if ((self->tele_head.datarate & 0x80) || (header[2] & 0x80))
 			trkh->recmode = 1;
 		else	trkh->recmode = 2;
 		trkh->filler = 0xF6;
@@ -443,7 +531,7 @@ dsk_err_t tele_open(DSK_DRIVER *s, const char *filename)
 
 		for (n = 0; n < header[0]; n++)
 		{
-			err = convert_sector(self, trkh, cyl, head, n);
+			err = convert_sector(self, trkh, cyl, head, n, diagfunc);
 			if (err)
 			{
 				ldbs_free(trkh);
