@@ -41,7 +41,7 @@ typedef struct
  * two platforms.
  * */
 
-dsk_err_t win32_open(DSK_DRIVER *self, const char *filename);
+dsk_err_t win32_open(DSK_DRIVER *self, const char *filename, DSK_REPORTFUNC diagfunc);
 dsk_err_t win32c_open(DSK_DRIVER *self, const char *filename);
 dsk_err_t win32_creat(DSK_DRIVER *self, const char *filename);
 dsk_err_t win32_close(DSK_DRIVER *self);
