@@ -703,6 +703,10 @@ LDPUBLIC32 dsk_err_t LDPUBLIC16  dg_cpm86geom(DSK_GEOMETRY *dg, const unsigned c
 }
 
 
+static int is_power_of_two(unsigned x)
+{
+	return (x != 0) && ((x & (x - 1)) == 0);
+}
 
 /* Interpret an Apricot superblock */
 LDPUBLIC32 dsk_err_t LDPUBLIC16 dg_aprigeom(DSK_GEOMETRY *self, const unsigned char *bootsect)
@@ -721,7 +725,7 @@ LDPUBLIC32 dsk_err_t LDPUBLIC16 dg_aprigeom(DSK_GEOMETRY *self, const unsigned c
 	/* [1.4.1] If sector size is not a reasonable value, this
 	 *         could be a non-Apricot disk that happens to have
 	 *         ASCII at the start of the boot sector */
-	if ((self->dg_secsize % 128) || (self->dg_secsize == 0)) 
+	if ((self->dg_secsize % 128) || (self->dg_secsize == 0) || !is_power_of_two(self->dg_secsize))
 		return DSK_ERR_BADFMT;
 	self->dg_secbase   = 1;
 	self->dg_heads     = bootsect[0x16];
@@ -751,21 +755,60 @@ LDPUBLIC32 dsk_err_t LDPUBLIC16 dg_aprigeom(DSK_GEOMETRY *self, const unsigned c
 LDPUBLIC32 dsk_err_t LDPUBLIC16  dg_opusgeom(DSK_GEOMETRY *dg, 
 		const unsigned char *bootsec)
 {
-	if (bootsec[0] != 0x18)	/* Z80 relative jump */
+	if (bootsec[0] != 0x18 || bootsec[1] != 0x05)	/* Z80 JR +5 */
+		return DSK_ERR_BADFMT;
+
+	/* [1.5.20] Add some sanity checks -- other kinds of boot sector may
+	 * start with a JR +5 but be jumping over something that isn't an Opus
+	 * disk geometry.
+	 *
+	 * Assume that the Z80 code in all Opus disks starts with 
+	 *	LD A,(HL)
+	 * 	LD (IX+0),A
+ 	 *
+	 * which is the case in the handful that I've examined, at least.
+	 */
+	if (bootsec[7] != 0x7E || bootsec[8] != 0xdd || bootsec[9] != 0x77 ||
+	    bootsec[10] != 0x00) 
+		return DSK_ERR_BADFMT;
+
+	/*
+	 * From the Opus Discovery Shadow ROM disassembly at 
+	 * <https://worldofspectrum.net/pub/sinclair/technical-docs/OpusDiscovery_ShadowROMv2.2Disassembly.html> 
+	 * the boot sector format is given as:
+	 *
+ 	 * 1C3E LOST_BLCK JR   1C45,GO_BOOT      Step over the databytes.
+	 *                DEFB +28               40 tracks.
+	 *                DEFB +12               18 sectors/track.
+	 *                DEFB +40               256 bytes a block, SS.
+	 *                DEFW +0000             Room for unique number.
+	 *
+	 * This corresponds to:
+	 *  bootsect[2] = cylinders
+	 *  bootsect[3] = sectors
+	 *  bootsect[4] bit 4    = heads
+	 *  bootsect[4] bits 6,7 = physical sector size
+	 *				00 => 128
+	 *				01 => 256
+	 * 				02 => 512
+	 *				03 => 1024
+	 *  bootsect[5-6] = unique ID.
+	 */
+	if (bootsec[2] > 85 || bootsec[3] > 32 || 
+	    bootsec[2] == 0 || bootsec[3] == 0)
 		return DSK_ERR_BADFMT;
 
 	dg->dg_cylinders = bootsec[2];
-	dg->dg_heads     = bootsec[3];
-	dg->dg_sectors   = bootsec[4];
+	dg->dg_sectors   = bootsec[3];
+	dg->dg_heads     = (bootsec[4] & 0x10) ? 2 : 1;
 	dg->dg_sidedness = SIDES_OUTOUT;	/* XXX Provisional */
 	dg->dg_secbase   = 1;
-	dg->dg_secsize   = 512;
 	dg->dg_datarate  = RATE_SD;
 	dg->dg_fm        = RECMODE_MFM;
 	dg->dg_nomulti   = 0;
 	dg->dg_rwgap     = 0x2A;		/* XXX Provisional */
 	dg->dg_fmtgap    = 0x52;		/* XXX Provisional */
-	dg->dg_secsize   = dsk_expand_psh(bootsec[4]);
+	dg->dg_secsize   = dsk_expand_psh(bootsec[4] >> 6);
 
 	return DSK_ERR_OK;
 }
