@@ -1,7 +1,7 @@
 /***************************************************************************
  *                                                                         *
  *    LIBDSK: General floppy and diskimage access library                  *
- *    Copyright (C) 2001  John Elliott <seasip.webmaster@gmail.com>            *
+ *    Copyright (C) 2001,2022  John Elliott <seasip.webmaster@gmail.com>   *
  *                                                                         *
  *    This library is free software; you can redistribute it and/or        *
  *    modify it under the terms of the GNU Library General Public          *
@@ -121,7 +121,10 @@ static dsk_err_t dsk_icreat(DSK_DRIVER **self, const char *filename, int ndrv, C
 
 
 /* Attempt to open a DSK file with driver <ndrv> */
-static dsk_err_t dsk_iopen(DSK_DRIVER **self, const char *filename, int ndrv, COMPRESS_DATA *cd)
+static dsk_err_t dsk_iopen(DSK_DRIVER **self, const char *filename, int ndrv, 
+			COMPRESS_DATA *cd,
+			DSK_REPORTFUNC diagfunc,
+			DSK_REPORTEND diagend)
 {
 	DRV_CLASS *dc = classes[ndrv];
 	dsk_err_t err;
@@ -135,8 +138,12 @@ static dsk_err_t dsk_iopen(DSK_DRIVER **self, const char *filename, int ndrv, CO
 	if (!*self) return DSK_ERR_NOMEM;
 	dr_construct(*self, dc);
 
-	err = (dc->dc_open)(*self, filename);
+	err = (dc->dc_open)(*self, filename, diagfunc);
 /*	printf("%s: open %s = %d\n", dc->dc_drvname, filename, err); */
+	if (diagend)
+	{
+		(*diagend)();
+	}
 	if (err == DSK_ERR_OK) 
 	{
 		(*self)->dr_compress = cd;
@@ -179,10 +186,21 @@ LDPUBLIC32 dsk_err_t LDPUBLIC16 dsk_creat(DSK_DRIVER **self, const char *filenam
 }
 
 
-
-/* Close a DSK file. Frees the pointer and sets it to null. */
-LDPUBLIC32 dsk_err_t LDPUBLIC16 dsk_open(DSK_DRIVER **self, const char *filename, const char *type,
+LDPUBLIC32 dsk_err_t LDPUBLIC16 dsk_open(DSK_DRIVER **self, 
+			const char *filename, 
+			const char *type,
 			const char *compress)
+{
+	return dsk_diagopen(self, filename, type, compress, NULL, NULL);
+}
+
+
+LDPUBLIC32 dsk_err_t LDPUBLIC16 dsk_diagopen(DSK_DRIVER **self, 
+			const char *filename, 
+			const char *type,
+			const char *compress,
+			DSK_REPORTFUNC diag,
+			DSK_REPORTEND diagend)
 {
 	int ndrv;
 	dsk_err_t e;
@@ -204,7 +222,8 @@ LDPUBLIC32 dsk_err_t LDPUBLIC16 dsk_open(DSK_DRIVER **self, const char *filename
 			{
 				if (match_drvname(type, classes[ndrv]))
 				{
-					e = dsk_iopen(self, filename, ndrv, cd);
+					e = dsk_iopen(self, filename, ndrv, cd,
+						diag, diagend);
 					if (e && cd) comp_abort(&cd);
 					return e;
 				}
@@ -215,7 +234,8 @@ LDPUBLIC32 dsk_err_t LDPUBLIC16 dsk_open(DSK_DRIVER **self, const char *filename
 	}
 	for (ndrv = 0; classes[ndrv]; ndrv++)
 	{
-		e = dsk_iopen(self, filename, ndrv, cd);
+		e = dsk_iopen(self, filename, ndrv, cd, 
+				diag, diagend);
 		if (e != DSK_ERR_NOTME) 
 		{
 			if (e != DSK_ERR_OK && cd) comp_abort(&cd);
