@@ -69,12 +69,15 @@ static DSK_NAMEDGEOM stdg[] =
 {"pcw1200", { SIDES_ALT,     80, 2,15,    1, 512, RATE_HD, 0x1B, 0x54,   RECMODE_MFM,  0 }, "PcW16 / IBM 1200k "}, /* 1.2M */
 {"mac400",  { SIDES_ALT,     80, 1,12,    1, 512, RATE_SD, 0x0C, 0x17,   RECMODE_GCR_MAC, 0 }, "Macintosh GCR 400k"}, /* Mac GCR 400k */
 {"mac800",  { SIDES_ALT,     80, 2,12,    1, 512, RATE_SD, 0x0C, 0x17,   RECMODE_GCR_MAC, 0 }, "Macintosh GCR 800k"}, /* Mac GCR 800k */
+{"c64_35",  { SIDES_ALT,     35, 1,21,    0, 256, RATE_SD, 0x12, 0x52,   RECMODE_GCR_C64, 0 }, "Commodore 64 GCR 170k"}, /* C64 35-track */
+{"c64_40",  { SIDES_ALT,     40, 1,21,    0, 256, RATE_SD, 0x12, 0x52,   RECMODE_GCR_C64, 0 }, "Commodore 64 GCR 192k"}, /* C64 40-track */
+{"c64cpm",  { SIDES_ALT,     34, 1,17,    0, 256, RATE_SD, 0x12, 0x52,   RECMODE_GCR_C64, 0 }, "Commodore 64 CP/M 132k"}, /* C64 35-track as CP/M container */
 
 /* Geometries below this line don't appear in dsk_format_t and can be accessed
  * only by name. */
 
 {"myz80",   { SIDES_ALT,     64, 1,128,   0,1024, RATE_ED, 0x2A, 0x52,   RECMODE_MFM,  0 }, "MYZ80 8Mb" }, /* MYZ80 8Mb */
-
+{"c64cpm",  { SIDES_ALT,     35, 1, 17,   0, 256, RATE_SD, 0x2A, 0x52,   RECMODE_GCR_C64, 0} , "Commodore 64 CP/M"}, 	/* Commodore 64 CP/M */
 /* re this comment ('This was commented out in libdsk-1.1.3, but I can't 
  * remember why. Bring it back.') -- I think I now remember why. The 
  * numeric format IDs were out of sync with the actual definitions, and I
@@ -361,6 +364,8 @@ dsk_err_t dg_parseline(char *linebuf, DSK_GEOMETRY *dg, char *description)
 	    dg->dg_fm = (dg->dg_fm & RECMODE_FLAGMASK) | RECMODE_FM;
         if (!strcmp(value, "mfm")) 
 	    dg->dg_fm = (dg->dg_fm & RECMODE_FLAGMASK) | RECMODE_MFM;
+        if (!strcmp(value, "gcr_c64")) 
+	    dg->dg_fm = (dg->dg_fm & RECMODE_FLAGMASK) | RECMODE_GCR_C64;
         if (!strcmp(value, "gcr_mac")) 
 	    dg->dg_fm = (dg->dg_fm & RECMODE_FLAGMASK) | RECMODE_GCR_MAC;
         if (!strcmp(value, "gcr_prodos")) 
@@ -368,7 +373,7 @@ dsk_err_t dg_parseline(char *linebuf, DSK_GEOMETRY *dg, char *description)
         if (!strcmp(value, "gcr_lisa")) 
 	    dg->dg_fm = (dg->dg_fm & RECMODE_FLAGMASK) | RECMODE_GCR_LISA;
         if (!strncmp(value, "gcr_mac_", 8) && sscanf(value + 8, "%d", &xrecmode)) 
-	    dg->dg_fm = ((dg->dg_fm & RECMODE_FLAGMASK) | (xrecmode & 0x1F) + RECMODE_GCR_FIRST);
+	    dg->dg_fm = ((dg->dg_fm & RECMODE_FLAGMASK) | ((xrecmode & 0x1F) + RECMODE_GCR_FIRST));
     }
     /* [1.4.1] 'Complement' flag */
     if (!strcmp(linebuf, "complement"))
@@ -428,6 +433,7 @@ dsk_err_t dg_store(FILE *fp, DSK_GEOMETRY *dg, char *description)
     {
         case RECMODE_MFM:        fprintf(fp, "recmode=MFM\n"); break;
         case RECMODE_FM:         fprintf(fp, "recmode=FM\n"); break;
+        case RECMODE_GCR_C64:    fprintf(fp, "recmode=GCR_C64\n"); break;
         case RECMODE_GCR_MAC:    fprintf(fp, "recmode=GCR_MAC\n"); break;
         case RECMODE_GCR_LISA:   fprintf(fp, "recmode=GCR_LISA\n"); break;
         case RECMODE_GCR_PRODOS: fprintf(fp, "recmode=GCR_PRODOS\n"); break;
