@@ -46,7 +46,8 @@
 #endif
 
 static dsk_format_t format = -1;
-static	char *intyp = NULL, *outtyp = NULL;
+static const char *intyp = NULL;
+static const char *outtyp = NULL;
 static	char *incomp = NULL, *outcomp = NULL;
 static	int inside = -1, outside = -1;
 static  int idstep =  0, odstep  =  0;
@@ -79,7 +80,7 @@ int check_numeric(char *arg, int *argc, char **argv)
 
 static void report(const char *s)
 {
-        fprintf(stderr, "%s\r", s);
+        fprintf(stderr, "%-79.79s\r", s);
         fflush(stderr);
 }
 
@@ -111,10 +112,8 @@ int help(int argc, char **argv)
 		               "\nDefault out-image type is DSK.\n\n");
 		
 	fprintf(stderr, "eg: %s /dev/fd0 myfile1.DSK\n"
-                        "    %s -side 1 /dev/fd0 myfile2.DSK\n" 
-                        "    %s -md3 /dev/fd0 md3boot.dsk\n" 
                         "    %s -otype floppy myfile.DSK /dev/fd0\n", 
-			AV0, AV0, AV0, AV0);
+			AV0, AV0);
 	return 1;
 }
 
@@ -141,12 +140,12 @@ int main(int argc, char **argv)
 	retries   = check_retry("-retry", &argc, argv);
 	if (present_arg("-idstep", &argc, argv)) idstep = 1;
 	if (present_arg("-odstep", &argc, argv)) odstep = 1;
-	if (!outtyp) outtyp = "dsk";
         format    = check_format("-format", &argc, argv);
         first     = check_numeric("-first", &argc, argv);
         last      = check_numeric("-last", &argc, argv);
 	args_complete(&argc, argv);
 	if (argc < 3) return help(argc, argv);
+	if (!outtyp) outtyp = guess_type(argv[2]);
 	return do_copy(argv[1], argv[2]);
 }
 
@@ -207,8 +206,14 @@ int do_copy(char *infile, char *outfile)
 		}
 abort:		;
 	}
-	if (indr)  dsk_close(&indr);
-	if (outdr) dsk_close(&outdr);
+	if (outdr) 
+	{
+		if (!e) e = dsk_close(&outdr); else dsk_close(&outdr);
+	}
+	if (indr) 
+	{
+		if (!e) e = dsk_close(&indr); else dsk_close(&indr);
+	}
 	printf("\r%-70.70s\n", "");
 	if (e)
 	{
