@@ -112,6 +112,78 @@ HRESULT CLibrary::GetBuffer(VARIANT *buf, unsigned char **bufout, unsigned lengt
 }
 
 
+static IReporter *gl_diagrep = NULL;
+
+void diag_on(const char *s)
+{
+	if (!gl_diagrep) return;
+	CComBSTR bs(s);
+	gl_diagrep->report(bs);
+}
+
+
+void diag_off(void)
+{
+	if (!gl_diagrep) return;
+	gl_diagrep->endreport();
+}
+
+STDMETHODIMP CLibrary::diag_open(BSTR f, IDispatch *reporter, BSTR d, BSTR c, IDisk **ppDisk)
+{
+	DSK_PDRIVER pd;
+	dsk_err_t err;
+	char *filename;
+	char *driver;
+	char *compress;
+
+	if (!f || !ppDisk)
+	{
+		return MapError(DSK_ERR_BADPTR);
+	}
+	// Check that the reporter callback is actually an IReporter.
+	IReporter *pReporter;
+	HRESULT hr;
+
+	hr = reporter->QueryInterface(IID_IReporter, (void **)&pReporter);
+	if (FAILED(hr)) return hr;
+
+	filename = get_string(f);
+	driver   = get_string(d);
+	compress = get_string(c);
+
+	if (driver[0] == 0)   
+	{
+		dsk_free(driver);
+		driver   = NULL;
+	}
+	if (compress[0] == 0) 
+	{
+		dsk_free(compress);
+		compress = NULL;
+	}
+	gl_diagrep = pReporter;
+	err = dsk_diagopen(&pd, filename, driver, compress, diag_on, diag_off);
+	gl_diagrep = NULL;
+	pReporter->Release();
+	if (filename) dsk_free(filename);
+	if (driver)   dsk_free(driver);
+	if (compress) dsk_free(compress);
+
+	if (err)
+	{
+		return MapError(err);
+	}
+	hr = CDisk::CreateInstance(ppDisk);
+	if (FAILED(hr)) return hr;
+
+	((CDisk *)ppDisk[0])->m_driver = pd;
+
+	return hr;
+}
+
+
+
+
 STDMETHODIMP CLibrary::open(BSTR f, BSTR d, BSTR c, IDisk **ppDisk)
 {
 	DSK_PDRIVER pd;
