@@ -31,7 +31,8 @@ class ATL_NO_VTABLE CLibrary :
 	public CComObjectRootEx<CComSingleThreadModel>,
 	public CComCoClass<CLibrary, &CLSID_Library>,
 	public ISupportErrorInfo,
-	public IDispatchImpl<ILibrary, &IID_ILibrary, &LIBID_LIBDSK>
+	public IDispatchImpl<ILibrary, &IID_ILibrary, &LIBID_LIBDSK>,
+	public IDispatchImpl<ILibrary2, &IID_ILibrary2, &LIBID_LIBDSK>
 {
 public:
 	CLibrary()
@@ -43,8 +44,8 @@ DECLARE_REGISTRY_RESOURCEID(IDR_LIBRARY)
 DECLARE_PROTECT_FINAL_CONSTRUCT()
 
 BEGIN_COM_MAP(CLibrary)
-	COM_INTERFACE_ENTRY(ILibrary)
-	COM_INTERFACE_ENTRY(IDispatch)
+	COM_INTERFACE_ENTRY(ILibrary2)
+//	COM_INTERFACE_ENTRY(IDispatch)
 	COM_INTERFACE_ENTRY(ISupportErrorInfo)
 END_COM_MAP()
 
@@ -69,6 +70,8 @@ public:
 	STDMETHOD(create)(/*[in,string]*/ BSTR filename, /*[in,string]*/ BSTR driver, /*[in,string]*/ BSTR compress, /*[out]*/ IDisk **ppDisk);
 	STDMETHOD(open)(/*[in,string]*/ BSTR filename, /*[in, string]*/ BSTR driver, /*[in,string]*/ BSTR compress, 
 		/*[out, result]*/ IDisk **ppDisk);
+	STDMETHOD(diag_open)(/*[in,string]*/ BSTR filename, /*[in]*/ IDispatch *reporter, /*[in, string]*/ BSTR driver, /*[in,string]*/ BSTR compress, 
+		/*[out, result]*/ IDisk **ppDisk);
 
 private:	
 	HRESULT MapError(int err);
